<?php

class Gozen_Growth_Automation
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Gozen_Growth_Automation_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if (defined('GOZENGROWTH_VERSION')) {
			$this->version = GOZENGROWTH_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'Gozen Growth';

		$this->load_dependencies();

		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Plugin_Name_Loader. Orchestrates the hooks of the plugin.
	 * - Plugin_Name_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-gozen-growth-automation-loader.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-gozen-growth-automation-public.php';


		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/env.php';

		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-gozen-growth-webhook.php';

		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-gozen-growth-utils.php';


		$this->loader = new Gozen_Growth_Automation_Loader();
	}


	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Gozn_Groth_Public($this->get_plugin_name(), $this->get_version());

		// register alll rest routs

		$this->loader->add_action('rest_api_init', $plugin_public, 'add_api_routes');


		// this will get called when an item is added to cart
		$this->loader->add_action('woocommerce_add_to_cart', $plugin_public, 'cart_updated_action', 10, 10);

		// this will get called when an item is removed from cart
		$this->loader->add_action('woocommerce_cart_item_removed', $plugin_public, 'cart_updated_action', 10, 10);


		// this will get called when a cart is emptied
		$this->loader->add_action('woocommerce_checkout_after_customer_details', $plugin_public, 'cart_updated_action', 10, 10);


		// this filter will add additional webhook topics 
		$this->loader->add_filter('woocommerce_webhook_topic_hooks', $plugin_public, 'add_new_topic_hooks', 10, 10);

		//this will add additionl events for webhooks
		$this->loader->add_filter('woocommerce_valid_webhook_events', $plugin_public, 'add_new_topic_events', 10, 10);

		//this will add additionl events for webhooks
		// $this->loader->add_filter('woocommerce_get_checkout_url', $plugin_public, 'make_checkout_url_global', 100, 10);

		// this will get called when a cart total is reseted
		// $this->loader->add_action('woocommerce_cart_reset', $plugin_public, 'cart_updated_action', 10, 10);

		// this will get called when a cart is emptied
		// $this->loader->add_action('woocommerce_cart_emptied', $plugin_public, 'cart_updated_action', 10, 10);


	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Gozen_Growth_Automation_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}
}